#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os
import dbus
import dbus.service

import gobject

from tacix.server.nx.nxutils import *
from tacix import util
from tacix.server.nx.apps import SessionRunner

logger = util.get_logger('txmanager', '/var/log/tacixmanager.log')

PROVIDER_BUS_NAME='org.tacix.nx'
PROVIDER_PATH='/org/tacix/nx/provider'
PROVIDER_INTERFACE='org.tacix.nx.provider'

class Provider( dbus.service.Object ):
    def __init__( self, tacix_manager, bus ):
        logger.debug("Starting NXProvider")
        #from dbus.mainloop.glib import DBusGMainLoop 
 	#DBusGMainLoop(set_as_default=True)

        # Getting the system dbus
        #self.bus = dbus.SystemBus( )
        self.bus = bus
        
        bus_name = dbus.service.BusName(PROVIDER_BUS_NAME,
                                        self.bus,
                                        do_not_queue=True)
        dbus.service.Object.__init__(self, bus_name=bus_name, object_path=PROVIDER_PATH)
        #self.loop = gobject.MainLoop()

        self.ck_manager = dbus.Interface (
                self.bus.get_object ('org.freedesktop.ConsoleKit', '/org/freedesktop/ConsoleKit/Manager'),
                'org.freedesktop.ConsoleKit.Manager')

        #self.dbus_info = dbus.Interface(self.bus.get_object('org.freedesktop.DBus',
        #                                '/org/freedesktop/DBus/Bus', False), 'org.freedesktop.DBus')

        self.tacix_manager = tacix_manager
        logger.debug("NXProvider Started")
 
    def __get_resumable_sessions(self, user, options):
        sessions = {}
        for session in self.tacix_manager._get_sessions_for_type('nx'):
            properties = session.get_properties()
            if 'persistent' in properties and properties['persistent'] == '0':
                continue
            if properties['user'] != user:
                continue
            if 'status' in options and options['status'].find(properties['state']) == -1:
                continue
            logger.debug("Adding session %s" % properties)
            sessions[session.get_property('display')] = properties
        return sessions

    def __start_sessions(self, users):
        server = SessionRunner(users)
        return server.run()


    #### DBUS Methods/Signals
    @dbus.service.method( dbus_interface=PROVIDER_INTERFACE,
                       in_signature='s', out_signature='a{ss}', sender_keyword='sender', connection_keyword='conn' )
    def GetNXProxyOptions( self, session_id, sender, conn ):
        '''
        Gets everything needed by nxproxy to run.
        This method is or test porpuse, invoking should be restricted.

        Example:
        dbus-send --system --dest=org.tacix.nx \
          --type=method_call --print-reply --reply-timeout=2000 \
          /org/tacix/nx/provider \
          org.tacix.nx.provider \
          string:"user"
        '''
        properties = self.tacix_manager._get_session(session_id).nxoptions.get_properties_secure()
        ret = parse_parameters_dict(properties)
        return ret
 
    @dbus.service.method( dbus_interface=PROVIDER_INTERFACE,
                       in_signature='as', out_signature='b', sender_keyword='sender', connection_keyword='conn' )
    def StartSessions( self, users, sender, conn ):
        '''
        Creates a new session and starts it on a hidden X server.
        This method is or test porpuse, invoking should be restricted.

        Example:
        dbus-send --system --dest=org.tacix.nx \
          --type=method_call --print-reply --reply-timeout=2000 \
          /org/tacix/nx/provider \
          org.tacix.nx.provider \
          string:"user"
        '''
        return self.__start_session(users)

    @dbus.service.method( dbus_interface=PROVIDER_INTERFACE,
                       in_signature='a{ss}', out_signature='a{sa{ss}}', sender_keyword='sender', connection_keyword='conn' )
    def GetResumableSessions(self, options, sender=None, conn=None):
        '''
        Example:
        dbus-send --system --dest=org.tacix \
          --type=method_call --print-reply --reply-timeout=2000 \
          /org/tacix/nx/provider \
          org.tacix.nx.provider.GetResumableSessions
        '''
        #return {}
        #self._check_polkit_privilege(sender, conn, 'org.tacix.sessionmanager.resume')
        return self.__get_resumable_sessions(options['user'], options)

    @dbus.service.method( dbus_interface=PROVIDER_INTERFACE,
                       in_signature='', out_signature='a{sa{ss}}', sender_keyword='sender', connection_keyword='conn' )
    def GetShadowableSessions(self, sender=None, conn=None):
        '''
        Example:
        dbus-send --system --dest=org.tacix \
          --type=method_call --print-reply --reply-timeout=2000 \
          /org/tacix/nx/provider \
          org.tacix.nx.provider.GetShadowableSessions
        '''

        #self._check_polkit_privilege(sender, conn, 'org.tacix.sessionmanager.resume')
        sessions = {}
        ck_sessions = self.ck_manager.GetSessions()
        for ck_op_session in ck_sessions:
            ck_session = dbus.Interface (self.bus.get_object ('org.freedesktop.ConsoleKit', ck_op_session), 'org.freedesktop.ConsoleKit.Session')
            display = ck_session.GetX11Display().replace(':', '')
            if display != '' and int(display) < 100:
                session_opts = {'display': display,
                    'type': 'console',
                    'uniqueid': '',
                    'cli_opts': '--------',
                    'depth': '',
                    'screen': 'x',
                    'available': 'Running',
                    'state': 'Running',
                    'session': 'X%s (Local)' % display}
                sessions[session_opts['display']] = session_opts

        for session in self.tacix_manager.manager_sessions.values():
            props = session.get_properties()
            props['available'] = 'Yes'
            props['session'] = props['uniqueid']
            sessions[session.get_property('display')] = props
        return sessions

    @dbus.service.method( dbus_interface=PROVIDER_INTERFACE,
                       in_signature='s', out_signature='', sender_keyword='sender', connection_keyword='conn' )
    def CreateShadowAuthority(self, ck_session_id, sender, conn):
        '''
        Example:
        dbus-send --system --dest=org.tacix \
          --type=method_call --print-reply --reply-timeout=2000 \
          /org/tacix/Manager \
          org.tacix.Manager.AskShadowPermission
        '''
        logger.debug('Asking for shadow permission')
        session = self.tacix_manager._get_session_for_ck_id (ck_session_id)
        #dialog = ShadowDialog (setup_func, callback, self.logger)
        #dialog.run()
        authority_file = session.AddUserAuthorization('nx')

        shadow_session = self.tacix_manager._get_session_for_session_worker(sender)
        shadow_session.nxoptions.set_property('shadowdisplay', session.nxoptions.get_property('display'))
        shadow_session.nxoptions.set_property('shadow_authority_file', authority_file)

        #session_worker = dbus.Interface (self.bus.get_object (sender, '/Session'), 'org.tacix.session.worker')
        #session_worker.SetShadowAuthority(authority_file)

    @dbus.service.method( dbus_interface=PROVIDER_INTERFACE,
                       in_signature='', out_signature='b', sender_keyword='sender', connection_keyword='conn' )
    def CreateShadow(self, user, sender, conn):
        '''
        Example:
        dbus-send --system --dest=org.tacix \
          --type=method_call --print-reply --reply-timeout=2000 \
          /org/tacix/Manager \
          org.tacix.Manager.CreateShadow
        '''
        file = xauth.create_xauthority_file_for_user(user, self.nxoptions.get_property('cookie'), self.nxoptions.get_property('display'))
        if file is None:
            raise TaciXSessionException('Could not create the authority file')
        return file
 

